@@ -142,10 +142,24 @@ class TestDXTabCompletion(unittest.TestCase):
         self.assert_completion("dx run my", "my applet ")
         self.assert_completion("dx ls ", "my applet ")
 
+        # not available to run when hidden
+        dxapplet.new(runSpec={"code": "placeholder", "interpreter": "bash"},
+                     dxapi="1.0.0",
+                     name="hidden",
+                     hidden=True)
+        self.assert_completion("dx ls hid", "hidden ")
+        self.assert_no_completions("dx run hid")
+
     def test_pipeline_completion(self):
         dxpipeline = dxpy.new_dxrecord(name="my workflow", types=["pipeline"])
         self.assert_completion("dx run my", "my workflow ")
 
+    def test_workflow_completion(self):
+        dxworkflow = dxpy.new_dxworkflow(name="my workflow")
+        self.assert_completion("dx run my", "my workflow ")
+        dxworkflow.hide()
+        self.assert_no_completions("dx run my")
+
     def test_project_completion(self):
         self.ids_to_destroy.append(dxpy.api.project_new({"name": "to select"})['id'])
         self.assert_completion("dx select to", "to select\\:")
