@@ -940,7 +940,8 @@ class TestDXAppletJob(unittest.TestCase):
     def tearDown(self):
         tearDownTempProjects(self)
 
-    def test_run_dxapplet(self):
+    @unittest.skipUnless(os.environ.get("DX_RUN_NEXT_TESTS"), "Temporarily skipping test that requires unreleased features")
+    def test_run_dxapplet_and_job_metadata(self):
         dxapplet = dxpy.DXApplet()
         dxapplet.new(name="test_applet",
                      dxapi="1.04",
@@ -980,6 +981,20 @@ def main():
         self.assertEqual(len(jobdesc["properties"]), 1)
         self.assertEqual(jobdesc["properties"]["$dnanexus_link.foo"], "barbaz")
 
+        # Test setting tags and properties on job
+        dxjob.add_tags(["foo", "bar", "foo"])
+        dxjob.set_properties({"foo": "bar", "$dnanexus.link": "thing"})
+        jobdesc = dxjob.describe()
+        self.assertEqual(sorted(jobdesc["tags"]), sorted(["foo", "bar", "$foo.bar"]))
+        self.assertEqual(jobdesc["properties"], {"foo": "bar",
+                                                 "$dnanexus.link": "thing",
+                                                 "$dnanexus_link.foo": "barbaz"})
+        dxjob.remove_tags(["bar", "baz"])
+        dxjob.set_properties({"$dnanexus.link": None})
+        jobdesc = dxjob.describe()
+        self.assertEqual(sorted(jobdesc["tags"]), sorted(["foo", "$foo.bar"]))
+        self.assertEqual(jobdesc["properties"], {"foo": "bar", "$dnanexus_link.foo": "barbaz"})
+
         # Test with fields parameter
         smaller_jobdesc = dxjob.describe(fields={"id": True, "state": True, "parentJob": True})
         self.assertEqual(len(smaller_jobdesc), 3)
@@ -994,8 +1009,6 @@ def main():
 
         dxjob.terminate()
 
-@unittest.skipUnless(testutil.TEST_RUN_JOBS,
-                     'skipping test that would run a job')
 class TestDXAnalysisWorkflow(unittest.TestCase):
     def setUp(self):
         setUpTempProjects(self)
@@ -1003,7 +1016,9 @@ class TestDXAnalysisWorkflow(unittest.TestCase):
     def tearDown(self):
         tearDownTempProjects(self)
 
-    def test_run_workflow(self):
+    @unittest.skipUnless(testutil.TEST_RUN_JOBS and os.environ.get("DX_RUN_NEXT_TESTS"),
+                         'skipping test that would run a job and temporarily skipping test that requires unreleased features')
+    def test_run_workflow_and_analysis_metadata(self):
         dxworkflow = dxpy.DXWorkflow(dxpy.api.workflow_new({"project": self.proj_id})['id'])
         dxapplet = dxpy.DXApplet()
         dxapplet.new(name="test_applet",
@@ -1027,6 +1042,257 @@ def main(number):
         dxjob = dxpy.DXJob(analysis_desc['stages'][0]['execution']['id'])
         self.assertEqual(dxjob.describe()['input'].get("number"), 32)
 
+        # Test setting tags and properties on analysis
+        dxanalysis.add_tags(["foo", "bar", "foo"])
+        dxanalysis.set_properties({"foo": "bar", "$dnanexus.link": "thing"})
+        analysis_desc = dxanalysis.describe()
+        self.assertEqual(sorted(analysis_desc["tags"]), sorted(["foo", "bar"]))
+        self.assertEqual(analysis_desc["properties"], {"foo": "bar", "$dnanexus.link": "thing"})
+        dxanalysis.remove_tags(["bar", "baz"])
+        dxanalysis.set_properties({"$dnanexus.link": None})
+        analysis_desc = dxanalysis.describe()
+        self.assertEqual(analysis_desc["tags"], ["foo"])
+        self.assertEqual(analysis_desc["properties"], {"foo": "bar"})
+
+    @unittest.skipUnless(os.environ.get("DX_RUN_NEXT_TESTS"), "Temporarily skipping test that requires unreleased features")
+    def test_new_dxworkflow(self):
+        blankworkflow = dxpy.new_dxworkflow()
+        self.assertIsInstance(blankworkflow, dxpy.DXAnalysisWorkflow)
+        desc = blankworkflow.describe()
+        self.assertEqual(desc['title'], blankworkflow.get_id())
+        self.assertEqual(desc['summary'], '')
+        self.assertEqual(desc['description'], '')
+        self.assertEqual(desc['outputFolder'], None)
+
+        dxworkflow = dxpy.new_dxworkflow(title='mytitle', summary='mysummary', description='mydescription', output_folder="/foo")
+        self.assertIsInstance(dxworkflow, dxpy.DXAnalysisWorkflow)
+        desc = dxworkflow.describe()
+        self.assertEqual(desc['title'], 'mytitle')
+        self.assertEqual(desc['summary'], 'mysummary')
+        self.assertEqual(desc['description'], 'mydescription')
+        self.assertEqual(desc['outputFolder'], '/foo')
+
+        secondworkflow = dxpy.new_dxworkflow(init_from=dxworkflow)
+        self.assertIsInstance(secondworkflow, dxpy.DXAnalysisWorkflow)
+        self.assertNotEqual(dxworkflow.get_id(), secondworkflow.get_id())
+        desc = secondworkflow.describe()
+        self.assertEqual(desc['title'], 'mytitle')
+        self.assertEqual(desc['summary'], 'mysummary')
+        self.assertEqual(desc['description'], 'mydescription')
+        self.assertEqual(desc['outputFolder'], '/foo')
+
+    @unittest.skipUnless(os.environ.get("DX_RUN_NEXT_TESTS"), "Temporarily skipping test that requires unreleased features")
+    def test_add_move_remove_stages(self):
+        dxworkflow = dxpy.new_dxworkflow()
+        dxapplet = dxpy.DXApplet()
+        dxapplet.new(dxapi="1.0.0",
+                     inputSpec=[{"name": "my_input", "class": "string"}],
+                     outputSpec=[],
+                     runSpec={"code": "", "interpreter": "bash"})
+        # Add stages
+        first_stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
+                                           stage_input={"my_input": "hello world"},
+                                           instance_type="dx_m1.large")
+        self.assertEqual(dxworkflow.editVersion, 1)
+        self.assertEqual(dxworkflow.stages[0]["name"], "stagename")
+        self.assertEqual(dxworkflow.stages[0]["folder"], "/outputfolder")
+        self.assertEqual(dxworkflow.stages[0]["input"]["my_input"], "hello world")
+        self.assertEqual(dxworkflow.stages[0]["systemRequirements"],
+                         {"*": {"instanceType": "dx_m1.large"}})
+        second_stage = dxworkflow.add_stage(dxapplet, folder="relativefolder",
+                                            instance_type={"main": "dx_m1.large", "foo": "dx_m1.medium"},
+                                            edit_version=1)
+        self.assertEqual(dxworkflow.editVersion, 2)
+        self.assertEqual(len(dxworkflow.stages), 2)
+        self.assertEqual(dxworkflow.stages[1]["executable"], dxapplet.get_id())
+        self.assertEqual(dxworkflow.stages[1]["folder"], "relativefolder")
+        self.assertEqual(dxworkflow.stages[1]["systemRequirements"],
+                         {"main": {"instanceType": "dx_m1.large"},
+                          "foo": {"instanceType": "dx_m1.medium"}})
+        with self.assertRaises(DXAPIError):
+            dxworkflow.add_stage(dxapplet, edit_version=1)
+
+        # Move stages
+        dxworkflow.move_stage(0, 1)
+        self.assertEqual(dxworkflow.editVersion, 3)
+        self.assertEqual(dxworkflow.stages[0]["id"], second_stage)
+        self.assertEqual(dxworkflow.stages[1]["id"], first_stage)
+        dxworkflow.move_stage(first_stage, 0, edit_version=3)
+        self.assertEqual(dxworkflow.editVersion, 4)
+        self.assertEqual(dxworkflow.stages[0]["id"], first_stage)
+        self.assertEqual(dxworkflow.stages[1]["id"], second_stage)
+
+        # Remove stages
+        removed_stage = dxworkflow.remove_stage(0)
+        self.assertEqual(removed_stage, first_stage)
+        self.assertEqual(dxworkflow.editVersion, 5)
+        self.assertEqual(len(dxworkflow.stages), 1)
+        self.assertEqual(dxworkflow.stages[0]["id"], second_stage)
+        with self.assertRaises(DXAPIError):
+            dxworkflow.remove_stage(first_stage) # should already have been removed
+        removed_stage = dxworkflow.remove_stage(second_stage, edit_version=5)
+        self.assertEqual(removed_stage, second_stage)
+        self.assertEqual(len(dxworkflow.stages), 0)
+        # bad input throws DXError
+        with self.assertRaises(DXError):
+            dxworkflow.remove_stage({})
+        with self.assertRaises(DXError):
+            dxworkflow.remove_stage(5)
+
+    def test_get_stage(self):
+        dxworkflow = dxpy.new_dxworkflow()
+        dxapplet = dxpy.DXApplet()
+        dxapplet.new(dxapi="1.0.0",
+                     inputSpec=[{"name": "my_input", "class": "string"}],
+                     outputSpec=[],
+                     runSpec={"code": "", "interpreter": "bash"})
+        # Add stages
+        first_stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
+                                           stage_input={"my_input": "hello world"})
+        second_stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
+                                            stage_input={"my_input": "hello world"})
+        # Get stages
+        stage_desc = dxworkflow.get_stage(0)
+        self.assertEqual(stage_desc['id'], first_stage)
+        stage_desc = dxworkflow.get_stage(first_stage)
+        self.assertEqual(stage_desc['id'], first_stage)
+        stage_desc = dxworkflow.get_stage(1)
+        self.assertEqual(stage_desc['id'], second_stage)
+        stage_desc = dxworkflow.get_stage(second_stage)
+        self.assertEqual(stage_desc['id'], second_stage)
+
+        # Errors
+        with self.assertRaises(DXError):
+            dxworkflow.get_stage(-1)
+        with self.assertRaises(DXError):
+            dxworkflow.get_stage(3)
+        with self.assertRaises(DXError):
+            dxworkflow.get_stage('foo')
+        with self.assertRaises(DXError):
+            dxworkflow.get_stage('stage-123456789012345678901234')
+
+    @unittest.skipUnless(os.environ.get("DX_RUN_NEXT_TESTS"), "Temporarily skipping test that requires unreleased features")
+    def test_update(self):
+        dxworkflow = dxpy.new_dxworkflow(title='title', summary='summary', description='description', output_folder='/foo')
+        self.assertEqual(dxworkflow.editVersion, 0)
+        for metadata in ['title', 'summary', 'description']:
+            self.assertEqual(getattr(dxworkflow, metadata), metadata)
+        self.assertEqual(dxworkflow.outputFolder, '/foo')
+
+        # update title, summary, description, outputFolder by value
+        dxworkflow.update(title='Title', summary='Summary', description='Description', output_folder='/bar/baz')
+        self.assertEqual(dxworkflow.editVersion, 1)
+        for metadata in ['title', 'summary', 'description']:
+            self.assertEqual(getattr(dxworkflow, metadata), metadata.capitalize())
+        self.assertEqual(dxworkflow.outputFolder, '/bar/baz')
+
+        # use unset_title
+        dxworkflow.update(unset_title=True, edit_version=1)
+        self.assertEqual(dxworkflow.editVersion, 2)
+        self.assertEqual(dxworkflow.title, dxworkflow.get_id())
+
+        # use unset_output_folder
+        dxworkflow.update(unset_output_folder=True, edit_version=2)
+        self.assertEqual(dxworkflow.editVersion, 3)
+        self.assertIsNone(dxworkflow.outputFolder)
+
+        # can't provide both title and unset_title=True
+        with self.assertRaises(DXError):
+            dxworkflow.update(title='newtitle', unset_title=True)
+        self.assertEqual(dxworkflow.editVersion, 3)
+
+        dxapplet = dxpy.DXApplet()
+        dxapplet.new(dxapi="1.0.0",
+                     inputSpec=[{"name": "my_input", "class": "string"}],
+                     outputSpec=[],
+                     runSpec={"code": "", "interpreter": "bash"})
+        stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
+                                     stage_input={"my_input": "hello world"})
+        self.assertEqual(dxworkflow.editVersion, 4)
+        self.assertEqual(dxworkflow.stages[0]["input"]["my_input"], "hello world")
+
+        # test stage modifications using update method
+        dxworkflow.update(summary='newsummary',
+                          stages={stage: {"folder": "/newoutputfolder",
+                                          "input": {"my_input": None}}})
+        self.assertEqual(dxworkflow.editVersion, 5)
+        self.assertEqual(dxworkflow.summary, 'newsummary')
+        self.assertEqual(dxworkflow.stages[0]["folder"], "/newoutputfolder")
+        self.assertNotIn("my_input", dxworkflow.stages[0]["input"])
+
+        # no-op update
+        dxworkflow.update()
+        self.assertEqual(dxworkflow.editVersion, 5)
+
+    @unittest.skipUnless(os.environ.get("DX_RUN_NEXT_TESTS"), "Temporarily skipping test that requires unreleased features")
+    def test_update_stage(self):
+        dxworkflow = dxpy.new_dxworkflow()
+        dxapplet = dxpy.DXApplet()
+        dxapplet.new(dxapi="1.0.0",
+                     inputSpec=[{"name": "my_input", "class": "string"}],
+                     outputSpec=[],
+                     runSpec={"code": "", "interpreter": "bash"})
+        # Add a stage
+        stage = dxworkflow.add_stage(dxapplet, name='stagename', folder="/outputfolder",
+                                     stage_input={"my_input": "hello world"},
+                                     instance_type={"main": "dx_m1.large"})
+        self.assertEqual(dxworkflow.editVersion, 1)
+        self.assertEqual(dxworkflow.stages[0]["executable"], dxapplet.get_id())
+        self.assertEqual(dxworkflow.stages[0]["name"], "stagename")
+        self.assertEqual(dxworkflow.stages[0]["folder"], "/outputfolder")
+        self.assertEqual(dxworkflow.stages[0]["input"]["my_input"], "hello world")
+        self.assertEqual(dxworkflow.stages[0]["systemRequirements"],
+                         {"main": {"instanceType": "dx_m1.large"}})
+
+        # Update just its metadata
+        dxworkflow.update_stage(stage, unset_name=True, folder="/newoutputfolder",
+                                stage_input={"my_input": None},
+                                instance_type="dx_m1.medium")
+        self.assertEqual(dxworkflow.editVersion, 2)
+        self.assertIsNone(dxworkflow.stages[0]["name"])
+        self.assertEqual(dxworkflow.stages[0]["folder"], "/newoutputfolder")
+        self.assertNotIn("my_input", dxworkflow.stages[0]["input"])
+        self.assertEqual(dxworkflow.stages[0]["systemRequirements"],
+                         {"*": {"instanceType": "dx_m1.medium"}})
+
+        # Update using stage index
+        dxworkflow.update_stage(0, folder="/", stage_input={"my_input": "foo"}, edit_version=2)
+        self.assertEqual(dxworkflow.editVersion, 3)
+        self.assertEqual(dxworkflow.stages[0]["folder"], "/")
+        self.assertEqual(dxworkflow.stages[0]["input"]["my_input"], "foo")
+
+        # no-op update
+        dxworkflow.update_stage(0)
+        self.assertEqual(dxworkflow.editVersion, 3)
+
+        # error when providing name and unset_name
+        with self.assertRaises(DXError):
+            dxworkflow.update_stage(0, name='foo', unset_name=True)
+        self.assertEqual(dxworkflow.editVersion, 3)
+
+        # Update its executable
+        second_applet = dxpy.DXApplet()
+        second_applet.new(dxapi="1.0.0",
+                          inputSpec=[{"name": "my_new_input", "class": "string"}],
+                          outputSpec=[],
+                          runSpec={"code": "", "interpreter": "bash"})
+
+        # Incompatible executable
+        try:
+            dxworkflow.update_stage(stage, executable=second_applet)
+            raise Exception("expected an error for updating a stage with an incompatible executable, but it succeeded")
+        except DXAPIError as e:
+            self.assertIsInstance(e, DXAPIError)
+            self.assertEqual(e.name, 'InvalidState')
+            self.assertEqual(dxworkflow.stages[0]["executable"], dxapplet.get_id())
+            self.assertEqual(dxworkflow.editVersion, 3)
+
+        # Successful update with force
+        dxworkflow.update_stage(stage, executable=second_applet, force=True)
+        self.assertEqual(dxworkflow.editVersion, 4)
+        self.assertEqual(dxworkflow.stages[0]["executable"], second_applet.get_id())
+        self.assertNotIn("my_input", dxworkflow.stages[0]["input"])
+
 @unittest.skipUnless(testutil.TEST_CREATE_APPS,
                      'skipping test that would create an app')
 class TestDXApp(unittest.TestCase):
@@ -1205,7 +1471,7 @@ class TestDXSearch(unittest.TestCase):
 
     @unittest.skipUnless(testutil.TEST_RUN_JOBS,
                          'skipping test that would run a job')
-    def test_find_jobs(self):
+    def test_find_executions(self):
         dxapplet = dxpy.DXApplet()
         dxapplet.new(name="test_applet",
                      dxapi="1.0.0",
@@ -1220,32 +1486,82 @@ class TestDXSearch(unittest.TestCase):
         dxrecord.close()
         prog_input = {"chromosomes": {"$dnanexus_link": dxrecord.get_id()},
                       "rowFetchChunk": 100}
-        dxjob = dxapplet.run(applet_input=prog_input, tags=["foo"], properties={"foo": "bar"})
-        results = list(dxpy.find_jobs(executable=dxapplet,
-                                      project=dxapplet.get_proj_id(),
-                                      origin_job=dxjob.get_id(),
-                                      parent_job='none',
-                                      tags=["foo"],
-                                      properties={"foo": True},
-                                      created_after='-15s',
-                                      describe=True))
-        self.assertEqual(len(results), 1)
-        result = results[0]
-        self.assertEqual(result["id"], dxjob.get_id())
-        self.assertTrue("describe" in result)
-        self.assertEqual(result["describe"]["id"], dxjob.get_id())
-        self.assertEqual(result["describe"]["class"], "job")
-        self.assertEqual(result["describe"]["applet"], dxapplet.get_id())
-        self.assertEqual(result["describe"]["project"], dxapplet.get_proj_id())
-        self.assertEqual(result["describe"]["originJob"], dxjob.get_id())
-        self.assertEqual(result["describe"]["parentJob"], None)
-        self.assertEqual(result["describe"]["tags"], ["foo"])
-        self.assertEqual(result["describe"]["properties"], {"foo": "bar"})
+        dxworkflow = dxpy.new_dxworkflow(name='find_executions test workflow')
+        stage = dxworkflow.add_stage(dxapplet, stage_input=prog_input)
+        dxanalysis = dxworkflow.run({stage+".rowFetchChunk": 200},
+                                    tags=["foo"],
+                                    properties={"foo": "bar"})
+        dxapplet.run(applet_input=prog_input)
+        dxjob = dxapplet.run(applet_input=prog_input, tags=["foo", "bar"], properties={"foo": "baz"})
 
-        results = list(dxpy.find_jobs(tags=["bar"]))
-        self.assertEqual(len(results), 0)
-        results = list(dxpy.find_jobs(properties={"foo": "baz"}))
-        self.assertEqual(len(results), 0)
+        # Wait for job to be created
+        executions = [stage['execution']['id'] for stage in dxanalysis.describe()['stages']]
+        t = 0
+        while len(executions) > 0:
+            try:
+                dxpy.api.job_describe(executions[len(executions) - 1], {})
+                executions.pop()
+            except DXAPIError:
+                t += 1
+                if t > 20:
+                    raise Exception("Timeout while waiting for job to be created for an analysis stage")
+                time.sleep(1)
+
+        me = None
+        common_conditions = {'executable': dxapplet,
+                             'project': dxapplet.get_proj_id(),
+                             'created_after': '-150s',
+                             'describe': True,
+                             'first_page_size': 1}
+        methods = (dxpy.find_executions, dxpy.find_analyses, dxpy.find_jobs)
+        queries = ({'conditions': {}, 'n_results': [4, 1, 3]},
+                   {'conditions': {'include_subjobs': False}, 'n_results': [4, 1, 3]},
+                   {'conditions': {'origin_job': dxjob.get_id(), 'parent_job': 'none'}, 'n_results': [1, 0, 1]},
+                   {'conditions': {'origin_job': dxjob.get_id(), 'no_parent_job': True}, 'n_results': [1, 0, 1]},
+                   {'conditions': {'root_execution': dxjob.get_id()}, 'n_results': [1, 0, 1]},
+                   {'conditions': {'root_execution': dxanalysis.get_id()}, 'n_results': [2, 1, 1]},
+                   {'conditions': {'parent_analysis': dxanalysis.get_id()}, 'n_results': [1, 0, 1]},
+                   {'conditions': {'no_parent_analysis': True}, 'n_results': [3, 1, 2]},
+                   {'conditions': {'name': 'test_applet'}, 'n_results': [3, 0, 3]},
+                   {'conditions': {'executable': dxworkflow}, 'n_results': [1, 1, 0]},
+                   {'conditions': {'name': 'find_executions test workflow'}, 'n_results': [1, 1, 0]},
+                   {'conditions': {'name': '?est_apple*', 'name_mode': 'glob'}, 'n_results': [3, 0, 3]},
+                   {'conditions': {'name': 'test_apples*', 'name_mode': 'glob'}, 'n_results': [0, 0, 0]},
+                   {'conditions': {'name': '[t]+est_apple.+', 'name_mode': 'regexp'}, 'n_results': [3, 0, 3]},
+                   {'conditions': {'name': 'test_apples.+', 'name_mode': 'regexp'}, 'n_results': [0, 0, 0]},
+                   {'conditions': {'tags': ['foo']}, 'n_results': [2, 1, 1]},
+                   {'conditions': {'tags': ['bar']}, 'n_results': [1, 0, 1]},
+                   {'conditions': {'properties': {'foo': True}}, 'n_results': [2, 1, 1]},
+                   {'conditions': {'properties': {'foo': 'baz'}}, 'n_results': [1, 0, 1]})
+
+        for query in queries:
+            for i in range(len(methods)):
+                conditions = dict(common_conditions, **query['conditions'])
+                conditions['launched_by'] = me
+                results = list(methods[i](**conditions))
+                if len(results) != query['n_results'][i]:
+                    raise Exception("Query " + json.dumps(query['conditions']) + " returned " + str(len(results)) + ", but " + str(query['n_results'][i]) + " were expected")
+                self.assertEqual(len(results), query['n_results'][i])
+                if len(results) > 0:
+                    result = results[0]
+                    self.assertTrue("describe" in result)
+                    if result['describe']['id'].startswith('analysis'):
+                        self.assertEqual(result["id"], dxanalysis.get_id())
+                    else:
+                        self.assertEqual(result["describe"]["class"], "job")
+                        self.assertEqual(result["describe"]["applet"], dxapplet.get_id())
+                        self.assertEqual(result["describe"]["project"], dxapplet.get_proj_id())
+                        self.assertEqual(result["describe"]["name"], 'test_applet')
+
+                    me = result["describe"]["launchedBy"]
+
+        bad_queries = [{'no_parent_job': True, 'parent_job': dxjob.get_id()},
+                       {'no_parent_analysis': True, 'parent_analysis': dxanalysis.get_id()},
+                       {'name': 'foo', 'name_mode': 'nonexistent'}]
+        for query in bad_queries:
+            for method in methods:
+                with self.assertRaises(DXError):
+                    method(**query)
 
 class TestPrettyPrint(unittest.TestCase):
     def test_string_escaping(self):
